local M = {
  'gruvbox-community/gruvbox',
  lazy = false,
  priority = 999,
}

function M.config()
  local function normalizeTerminal() -- {{{
    vim.g.terminal_color_0 = 0
    vim.g.terminal_color_1 = 1
    vim.g.terminal_color_2 = 2
    vim.g.terminal_color_3 = 3
    vim.g.terminal_color_4 = 4
    vim.g.terminal_color_5 = 5
    vim.g.terminal_color_6 = 6
    vim.g.terminal_color_7 = 7
    vim.g.terminal_color_8 = 8
    vim.g.terminal_color_9 = 9
    vim.g.terminal_color_10 = 10
    vim.g.terminal_color_11 = 11
    vim.g.terminal_color_12 = 12
    vim.g.terminal_color_13 = 13
    vim.g.terminal_color_14 = 14
    vim.g.terminal_color_15 = 15
  end                                        -- }}}

  local function normalizeCmpKindHighlight() -- {{{
    vim.api.nvim_set_hl(0, 'CmpItemKindClass', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindConstant', { link = 'Constant' })
    vim.api.nvim_set_hl(0, 'CmpItemKindConstructor', { link = 'Special' })
    vim.api.nvim_set_hl(0, 'CmpItemKindEnum', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindEnumMember', { link = 'String' })
    vim.api.nvim_set_hl(0, 'CmpItemKindField', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindFile', { link = 'File' })
    vim.api.nvim_set_hl(0, 'CmpItemKindFolder', { link = 'Directory' })
    vim.api.nvim_set_hl(0, 'CmpItemKindFunction', { link = 'Function' })
    vim.api.nvim_set_hl(0, 'CmpItemKindInterface', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindKeyword', { link = 'Keyword' })
    vim.api.nvim_set_hl(0, 'CmpItemKindMethod', { link = 'Function' })
    vim.api.nvim_set_hl(0, 'CmpItemKindModule', { link = 'Include' })
    vim.api.nvim_set_hl(0, 'CmpItemKindProperty', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindSnippet', { link = 'Special' })
    vim.api.nvim_set_hl(0, 'CmpItemKindStruct', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindText', { link = 'String' })
    vim.api.nvim_set_hl(0, 'CmpItemKindTypeParameter', { link = 'Identifier' })
    vim.api.nvim_set_hl(0, 'CmpItemKindValue', { link = 'String' })
    vim.api.nvim_set_hl(0, 'CmpItemKindVariable', { link = 'Identifier' })
  end -- }}}

  local function copyFg(to, from, opts)
    local cterm = vim.api.nvim_get_hl_by_name(from, false)
    local gui = vim.api.nvim_get_hl_by_name(from, true)
    local o = vim.tbl_extend('force', { ctermfg = cterm.foreground, fg = gui.foreground }, opts or {})
    vim.api.nvim_set_hl(0, to, o)
  end

  local function clear(hl)
    vim.api.nvim_set_hl(0, hl, {})
  end

  local function link(hl, to)
    vim.api.nvim_set_hl(0, hl, { link = to })
  end

  local function normalizeTSHighlight() --- {{{
    link('@class', 'Type')
    link('@interface', 'Type')
    link('@namespace', 'Type')
    link('@event', 'Type')

    link('@type.qualifier', 'Keyword')
    link('@storageclass', 'Keyword')
    link('@modifier', 'Keyword')

    link('@attribute', 'PreProc')

    link('@tag.attribute', 'Identifier')
    link('@tag.delimiter', 'Delimiter')
  end                                        -- }}}

  local function normalizeTroubleHighlight() -- {{{
    copyFg('TroubleText', 'Normal')
    copyFg('TroubleFoldIcon', 'CursorLineNr')
    copyFg('TroubleLocation', 'LineNr')
    copyFg('TroubleIndent', 'LineNr')
    copyFg('TroubleSignInformation', 'DiagnosticSignInfo')
    -- TroubleCount   xxx links to TabLineSel
    -- copyFg('TroubleError', 'DiagnosticError')
    link('TroubleSignError', 'DiagnosticError')
    link('TroubleSignWarning', 'DiagnosticWarn')
    link('TroubleSignHint', 'DiagnosticHint')

    vim.api.nvim_set_hl(0, 'TroubleNormal', {})
  end -- }}}

  local function customLinks()
    copyFg('Function', 'Normal')
    copyFg('Operator', 'Normal')
  end

  local troublegr = vim.api.nvim_create_augroup('TroubleCursorline', { clear = true })
  vim.api.nvim_create_autocmd('WinEnter', {
    group = troublegr,
    pattern = '*Trouble',
    callback = function()
      vim.wo.cursorline = true
    end,
  })
  vim.api.nvim_create_autocmd('WinLeave', {
    group = troublegr,
    pattern = '*Trouble',
    callback = function()
      vim.wo.cursorline = false
    end,
  })

  local augr = vim.api.nvim_create_augroup('config_colorscheme', {});
  vim.api.nvim_create_autocmd('Colorscheme', {
    group = augr,
    pattern = '*',
    callback = function()
      normalizeTSHighlight()
      customLinks()
      normalizeTerminal()
      normalizeCmpKindHighlight()
      normalizeTroubleHighlight()
    end,
  })

  vim.api.nvim_create_autocmd('Colorscheme', {
    group = augr,
    pattern = 'gruvbox',
    callback = function()
      local gr = require('configs.colors')

      link('Directory', 'GruvboxGreen')
      link('TelescopeSelection', 'GruvboxOrangeSign')
      link('TelescopeMatching', 'GruvboxAqua')

      link('NeoTreeGitAdded', 'GruvboxGreen')
      link('NeoTreeGitDeleted', 'GruvboxRed')
      link('NeoTreeGitModified', 'GruvboxOrange')

      link('CmpItemAbbrMatch', 'GruvboxAquaBold')
      link('CmpItemAbbrMatchFuzzy', 'GruvboxAquaBold')

      link('FloatBorder', 'GruvBoxFg1')

      link('NonText', 'GruvboxBg3') -- make it more bright

      local linenr_style = { 'underdotted' }
      if vim.fn.has('nvim-0.9') then
        linenr_style = {}
      end

      -- link('@tag.delimiter.xml', 'xmlTag')
      -- link('@tag.xml', 'xmlTagName')
      -- link('@tag.attribute.xml', 'xmlAttrib')
      --
      -- link('@tag.delimiter.html', 'htmlTag')
      -- link('@tag.html', 'htmlTagName')
      -- link('@tag.attribute.html', 'htmlArg')

      gr:hl('LineNr', 'gray', 'dark0_hard', linenr_style)
      gr:hl('LspCodeLens', 'gray', nil, { 'italic' })

      gr:hl('IndentBlanklineContextChar', 'orange', nil, { 'nocombine' })

      link('DiagnosticSignOk', 'GruvboxGreenSign')

      gr:hl('DiagnosticVirtualTextError', 'error', nil, { 'italic' })
      gr:hl('DiagnosticVirtualTextWarn', 'warn', nil, { 'italic' })
      gr:hl('DiagnosticVirtualTextInfo', 'info', nil, { 'italic' })
      gr:hl('DiagnosticVirtualTextHint', 'hint', nil, { 'italic' })
      gr:hl('DiagnosticVirtualTextOk', 'ok', nil, { 'italic' })

      -- gr:hl('GitSignsAddLine', nil, 'faded_green', {})
      -- vim.fn.sign_define('GitSignsAdd', { text = '│', texthl = 'GitSignsAdd', linehl = 'GitSignsAddLine', culhl='CursorLine' })
    end,
  })

  vim.api.nvim_create_autocmd('ColorschemePre', {
    group = augr,
    pattern = '*',
    callback = function()
      vim.fn.sign_define('DiagnosticSignError', { text = ' ', texthl = 'DiagnosticSignError' })
      vim.fn.sign_define('DiagnosticSignWarn', { text = ' ', texthl = 'DiagnosticSignWarn' })
      vim.fn.sign_define('DiagnosticSignInfo', { text = ' ', texthl = 'DiagnosticSignInfo' })
      vim.fn.sign_define('DiagnosticSignHint', { text = '', texthl = 'DiagnosticSignHint' })
    end,
  })

  local term = vim.env.TERM
  if (term ~= 'linux' or vim.g.neoray)
  then
    vim.o.termguicolors = true
    vim.g.gruvbox_contrast_dark = 'hard'
    vim.cmd.colorscheme('gruvbox')
  end
end

return M
